<?php

/**
 * _ntm_m_get_names().
 * _ntm_t_get_names().
 */
function _ntm_m_get_names() {
  $names = menu_get_menus();
  return $names;
}

function _ntm_t_get_names() {
  $options = module_exists('taxonomy') ? taxonomy_vocabulary_get_names() : NULL;
  $names = array();
  if ($options) {
    foreach ($options as $item) {
      $names[$item->vid] = $item->name;
    }
  }
  return $names;
}


/**
 * _ntm_m_get_flat_tree().
 * _ntm_t_get_flat_tree().
 */
function _ntm_m_get_flat_tree($source_id, $iid = 0, $max_depth = NTM_MAX_DEPTH) {
  if ($iid) {
    $root_info = menu_link_load($iid);
  }
  $cache = &drupal_static(__FUNCTION__, array());
  $cache_id = $source_id.$iid.$max_depth;
  if (!isset($cache[$cache_id])) {
    $cache[$cache_id] = menu_build_tree($source_id, array('max_depth' => (isset($root_info) ? $root_info['depth'] + $max_depth : $max_depth)));
  }
  $tree = $cache[$cache_id];
  if ($iid) {
    $tree = _ntm_m_get_sub_tree($tree, $iid);
    $tree = $tree['below'];
  }
  if ($tree) {
    $tree = _ntm_m_items_collect(menu_tree_output($tree));
 /* depth correct */
    if (isset($root_info)) {
      foreach ($tree as $item) {
        $item->depth-= $root_info['depth'];
      }
    }
 /* active trail */
    $menu_active_trail = menu_get_active_trail();
    $active_item = end($menu_active_trail);
    if (isset($active_item['mlid'])) {
      if ($active_parents = _ntm_m_get_pids($active_item['mlid'])) {
        foreach ($tree as $item) {
          $item->active_trail = (int)isset($active_parents[$item->iid]);
        }
      }
    }
  }
  return $tree;
}

function _ntm_t_get_flat_tree($source_id, $iid = 0, $max_depth = NTM_MAX_DEPTH) {
  if (module_exists('taxonomy')) {
    if (module_exists('i18n_taxonomy')) {
      $i18n_mode = i18n_taxonomy_vocabulary_mode($source_id);
      switch ($i18n_mode) {
        case I18N_MODE_NONE:
        case I18N_MODE_LOCALIZE: $tree = taxonomy_get_tree($source_id, $iid, $max_depth); break;
        case I18N_MODE_TRANSLATE:
        case I18N_MODE_LANGUAGE: $tree = i18n_taxonomy_get_tree($source_id, $GLOBALS['language']->language, $iid, $max_depth); break;
      }
    } else {
      $tree = taxonomy_get_tree($source_id, $iid, $max_depth);
    }
    foreach ($tree as $term) {
      $term->depth++;
      $term->iid = $term->tid;
      $term->url = "taxonomy/term/$term->iid";
      $term->title = isset($i18n_mode) && $i18n_mode == I18N_MODE_LOCALIZE ? i18n_taxonomy_term_name($term, $GLOBALS['language']->language) : $term->name;
    }
 /* active trail */
    if (arg(0) == 'taxonomy' && arg(1) == 'term' && is_numeric(arg(2))) {
      $active_id = arg(2);
      if ($active_parents = _ntm_t_get_pids($active_id)) {
        foreach ($tree as $item) {
          $item->active_trail = (int)isset($active_parents[$item->iid]);
        }
      }
    }
  }
  return $tree;
}


/**
 * _ntm_m_get_pids().
 * _ntm_t_get_pids().
 */
function _ntm_m_get_pids($c_pid) {
  $pids = array();
  while (TRUE) {
    $c_pid = db_select('menu_links', 'm')->fields('m', array('plid'))->condition('m.mlid', $c_pid)->execute()->fetchField();
    if ($c_pid) $pids[$c_pid] = $c_pid; else break;
  }
  return $pids;
}

function _ntm_t_get_pids($c_pid) {
  $pids = array();
  while (TRUE) {
    $c_pid = db_select('taxonomy_term_hierarchy', 't')->fields('t', array('parent'))->condition('t.tid', $c_pid)->execute()->fetchField();
    if ($c_pid) $pids[$c_pid] = $c_pid; else break;
  }
  return $pids;
}


/**
 * _ntm_flat_tree_render().
 */
function _ntm_flat_tree_render($items, $decor, $style, $wrappers) {
  $tree = array();
  foreach ($items as $index => $c_item) {
    $wrapper_level_name = $c_item->depth == 1 ? 'level_1' : 'level_2';
    $wr_opn = isset($wrappers[$wrapper_level_name]) ? '<'. implode('><',  $wrappers[$wrapper_level_name]).'>' : '';
    $wr_cls = isset($wrappers[$wrapper_level_name]) ? '</'.implode('></', array_reverse($wrappers[$wrapper_level_name])).'>' : '';
    $class = array();
    if (isset($c_item->active_trail) && $c_item->active_trail == 1) $class[]= 'active_trail';
    $new_item = array('data' => l("$wr_opn$c_item->title$wr_cls", $c_item->url, array('html' => TRUE)), 'children' => array(), 'class' => $class);
    if ($c_item->depth == 1) {
      $tree[]= $new_item;
    } else {
      _ntm_item_list_add_child($tree, $new_item, $c_item->depth-1);
    }
  }
  $item_list_html = theme('ntm_tree', array('tree' => $tree));
  return '<div class="menu-ntm menu-ntm-style-'.$style.' menu-ntm-decor-'.$decor.' menu-ntm-decor-'.$decor.'-style-'.$style.' clearfix">'.
            $item_list_html.
         '</div>';
}

function theme_ntm_tree($variables) {
  $tree = $variables['tree'];
  $output = '';
  if (count($tree)) {
    $output.= '<ul>';
    foreach ($tree as $item) {
      $level_output = $item['data'];
      $class = $item['class'];
      if (count($item['children']))                                  $level_output.= theme_ntm_tree(array('tree' => $item['children']));
      if (count($item['children']))                                  $class[]= 'has_child';
      if ($item == reset($tree))                                     $class[]= 'first';
      if ($item == end($tree))                                       $class[]= 'last';
      if (in_array('active_trail', $class) && $item == reset($tree)) $class[]= 'active_trail-first';
      if (in_array('active_trail', $class) && $item == end($tree))   $class[]= 'active_trail-last';
      $output.= '<li'.(count($class) ? ' class="'.implode(' ', $class).'"' : '').'>'.$level_output."</li>\n";
    }
    $output.= '</ul>';
  }
  return $output;
}


/**
 * support funcs
 */
function _ntm_item_list_add_child(&$elements, $child, $depth) {
  if ($depth > 1) {
    _ntm_item_list_add_child($elements[count($elements)-1]['children'], $child, $depth - 1);
  } else if ($depth == 1) {
    $elements[count($elements)-1]['children'][]= $child;
  }
}

function _ntm_m_get_sub_tree($tree, $iid) {
  foreach ($tree as $node) {
    if ($node['link'] && $node['link']['mlid'] == $iid) return $node;
    if ($node['below']) {
      $node = _ntm_m_get_sub_tree($node['below'], $iid);
      if ($node) return $node;
    }
  }
}

function _ntm_m_items_collect($tree, &$info = array()) {
  foreach ($tree as $node) {
    if (isset($node['#original_link'])) {
      $link = $node['#original_link'];
      $info[]= (object)array(
        'iid' => $link['mlid'],
        'title' => $link['title'],
        'url' => $link['href'],
        'depth' => $link['depth'],
      );
    }
    if (isset($node['#below'])) {
      _ntm_m_items_collect($node['#below'], $info);
    }
  }
  return $info;
}